# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
import requests

from .base_ts_message import BaseTsAppMessage
from contrast.utils.decorators import fail_loudly
from contrast_vendor import structlog as logging

logger = logging.getLogger("contrast")


class ObservedRoute(BaseTsAppMessage):
    def __init__(self, observed_route):
        # This message does not need "Application-Path" header but it doesn't hurt
        # either.
        super().__init__()
        self.base_url = f"{self.settings.api_url}/agents/v1.0/"

        self.body = {
            "signature": observed_route.signature,
            "verb": observed_route.verb,
            "url": observed_route.url,
            "sources": self._parse_sources(observed_route.sources),
        }

        session_id = self.settings.config.session_id
        if session_id:
            self.body.update({"session_id": session_id})

    @property
    def name(self):
        return "observed-route"

    @property
    def path(self):
        return (
            f"routes/{self.server_name_b64}/{self.server_path_b64}"
            f"/{self.server_type_b64}/{self.app_language_b64}/{self.app_name_b64}"
            "/observed"
        )

    @property
    def request_method(self):
        return requests.post

    @fail_loudly("Failed to process ObservedRoute response")
    def process_response(self, response, reporting_client):
        self.process_response_code(response, reporting_client)

    def _parse_sources(self, sources):
        new_sources = []
        for source in sources:
            try:
                new_sources.append(dict(type=source.type, name=source.name))
            except Exception:
                logger.debug("Could not parse source %s", source)

        return new_sources



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.

from .config_builder import ConfigBuilder
from .config_option import ConfigOption
from contrast.utils.configuration_utils import str_to_bool
from contrast.utils.loggers import (
    DEFAULT_PROGNAME,
    DEFAULT_SECURITY_LOG_PATH,
    DEFAULT_SECURITY_LOG_LEVEL,
    DEFAULT_LOG_PATH,
    DEFAULT_LOG_LEVEL,
)


class Agent(ConfigBuilder):
    def __init__(self):
        super().__init__()

        self.default_options = [
            # Some logger default values are handled by the logger
            ConfigOption(
                canonical_name="agent.logger.level",
                default_value=DEFAULT_LOG_LEVEL,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="agent.logger.path",
                default_value=DEFAULT_LOG_PATH,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="agent.logger.stdout",
                default_value=False,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.logger.stderr",
                default_value=False,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.logger.progname",
                default_value=DEFAULT_PROGNAME,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="agent.security_logger.path",
                default_value=DEFAULT_SECURITY_LOG_PATH,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="agent.security_logger.level",
                default_value=DEFAULT_SECURITY_LOG_LEVEL,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="agent.python.enable_sys_monitoring",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.rewrite",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.disable_policy_rewrites",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.enable_automatic_middleware",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.enable_drf_response_analysis",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.enable_rep",
                default_value=True,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.enable_profiler",
                default_value=False,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.python.assess.use_pure_python_hooks",
                default_value=False,
                type_cast=str_to_bool,
            ),
            ConfigOption(
                canonical_name="agent.polling.app_activity_ms",
                default_value=30000,
                type_cast=int,
            ),
            ConfigOption(
                canonical_name="agent.polling.server_settings_ms",
                default_value=30_000,
                type_cast=int,
            ),
            ConfigOption(
                canonical_name="agent.polling.heartbeat_ms",
                default_value=30_000,
                type_cast=int,
            ),
        ]



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.

from .agent import Agent
from .api import Api
from .application import Application
from .assess import Assess
from .config_builder import ConfigBuilder
from .config_option import ConfigOption
from .agent_config import AgentConfig
from .inventory import Inventory
from .protect import Protect
from .root import Root
from .server import Server



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
import re

from contrast.agent.protect.rule.base_rule import BaseRule


class CmdInjection(BaseRule):
    """
    Command Injection Protection rule
    """

    RULE_NAME = "cmd-injection"

    BIN_SH_C = "/bin/sh-c"

    START_IDX = "start_idx"
    END_IDX = "end_idx"

    def find_attack(self, candidate_string=None, **kwargs):
        command_string = str(candidate_string) if candidate_string else None

        return super().find_attack(command_string, **kwargs)

    def build_sample(self, evaluation, command, **kwargs):
        sample = self.build_base_sample(evaluation)

        if command is not None:
            sample.details["command"] = command

        if self.START_IDX in kwargs or self.END_IDX in kwargs:
            sample.details["startIndex"] = kwargs.get(self.START_IDX, 0)
            sample.details["endIndex"] = kwargs.get(self.END_IDX, 0)
        elif command is not None:
            search_value = evaluation.value

            match = re.search(search_value, command, re.IGNORECASE)

            if match:
                sample.details["startIndex"] = match.start()
                sample.details["endIndex"] = match.end()

        return sample

    def infilter_kwargs(self, user_input, patch_policy):
        return dict(method=patch_policy.method_name, original_command=user_input)

    def skip_protect_analysis(self, user_input, args, kwargs):
        """
        cmdi rule supports list user input as well as str and bytes
        Do not skip protect analysis if user input is a  populated list
        """
        if isinstance(user_input, list) and user_input:
            return False

        return super().skip_protect_analysis(user_input, args, kwargs)

    def convert_input(self, user_input):
        if isinstance(user_input, list):
            user_input = " ".join(user_input)

        return super().convert_input(user_input)

    def infilter(self, match_string, **kwargs):
        # TODO: PYT-3088
        #  deserialization_rule = Settings().protect_rules[Deserialization.RULE_NAME]
        #  deserialization_rule.check_for_deserialization()
        pass

        super().infilter(match_string, **kwargs)



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.

from .config_builder import ConfigBuilder
from .config_option import ConfigOption
from contrast import AGENT_CURR_WORKING_DIR


class Api(ConfigBuilder):
    def __init__(self):
        super().__init__()

        self.default_options = [
            ConfigOption(canonical_name="api.url", default_value="", type_cast=str),
            ConfigOption(
                canonical_name="api.service_key",
                default_value="",
                type_cast=str,
                redacted=True,
            ),
            ConfigOption(
                canonical_name="api.api_key",
                default_value="",
                type_cast=str,
                redacted=True,
            ),
            ConfigOption(
                canonical_name="api.user_name",
                default_value="",
                type_cast=str,
                redacted=True,
            ),
            ConfigOption(
                canonical_name="api.request_audit.enable",
                default_value=False,
                type_cast=bool,
            ),
            ConfigOption(
                canonical_name="api.request_audit.path",
                default_value=AGENT_CURR_WORKING_DIR,
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="api.request_audit.requests",
                default_value=False,
                type_cast=bool,
            ),
            ConfigOption(
                canonical_name="api.request_audit.responses",
                default_value=False,
                type_cast=bool,
            ),
            ConfigOption(
                canonical_name="api.certificate.enable",
                default_value=False,
                type_cast=bool,
            ),
            ConfigOption(
                canonical_name="api.certificate.ignore_cert_errors",
                default_value=False,
                type_cast=bool,
            ),
            ConfigOption(
                canonical_name="api.certificate.ca_file",
                default_value="",
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="api.certificate.cert_file",
                default_value="",
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="api.certificate.key_file",
                default_value="",
                type_cast=str,
            ),
            ConfigOption(
                canonical_name="api.proxy.enable", default_value=False, type_cast=bool
            ),
            ConfigOption(
                canonical_name="api.proxy.url",
                default_value="",
                type_cast=str,
                redacted=True,
            ),
        ]



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.

ENVIRONMENT_VARIABLE_SRC = "ENVIRONMENT_VARIABLE"
USER_CONFIGURATION_FILE_SRC = "USER_CONFIGURATION_FILE"
DEFAULT_VALUE_SRC = "DEFAULT_VALUE"
CONTRAST_UI_SRC = "CONTRAST_UI"


class ConfigOption:
    def __init__(self, canonical_name, default_value, type_cast, redacted=False):
        self.canonical_name = canonical_name
        self.default_value = default_value
        self.type_cast = type_cast
        self.name = None
        self.override_value = None
        self.env_value = None
        self.file_value = None
        self.file_source = None
        self.ui_value = None
        self.redacted = redacted

    def value(self):
        if self.override_value is not None:
            return self.override_value
        if self.env_value is not None:
            return self.env_value
        if self.file_value is not None:
            return self.file_value
        if self.ui_value is not None:
            return self.ui_value
        return self.default_value

    def source(self):
        if self.env_value is not None:
            return ENVIRONMENT_VARIABLE_SRC
        if self.file_value is not None:
            return USER_CONFIGURATION_FILE_SRC
        if self.ui_value is not None:
            return CONTRAST_UI_SRC
        return DEFAULT_VALUE_SRC

    def provided_name(self):
        if self.name is not None:
            return self.name
        return self.canonical_name

    def loggable_value(self):
        return self.to_string(self.value())

    def to_string(self, raw_value):
        # If the value is empty, just return empty String.
        if raw_value in (None, ""):
            return ""
        # If the option is sensitive, like an API credential, we do not log or report it.
        if self.redacted:
            return "**REDACTED**"
        str_value = str(raw_value)
        # If the option is an enum, we need to clean it up.
        if str_value and str_value.startswith("ProtectionRule."):
            return str_value.lstrip("ProtectionRule.").lower()
        return str_value

    def clear(self):
        """
        This is used as a convenience method during testing to ensure a clean slate.
        """
        self.override_value = None
        self.env_value = None
        self.file_value = None
        self.ui_value = None
        self.file_source = None



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
import sys
import logging

import contrast
from contrast.assess_extensions import cs_str
from contrast.utils.loggers.structlog import init_structlog
from contrast.utils.configuration_utils import get_hostname
from contrast.utils.namespace import Namespace
from contrast.configuration.config_option import CONTRAST_UI_SRC, DEFAULT_VALUE_SRC

from contrast_vendor import structlog

from . import (
    DEFAULT_LOG_LEVEL,
    DEFAULT_PROGNAME,
    LOGGER_NAME,
    SECURITY_LOGGER_NAME,
)

STDOUT = "STDOUT"
STDERR = "STDERR"


class module(Namespace):
    initialized: bool = False


def setup_basic_agent_logger(level=logging.INFO):
    """
    Setup a logger without any user-supplied configuration, with defaults:
        1. log to stdout
        2. log in INFO level
        3. with default progname

    The logger created here is expected to be overridden with config values
    provided later on in the middleware creation cycle.
    """
    if not module.initialized:
        logger = logging.getLogger(LOGGER_NAME)
        logger.addHandler(logging.StreamHandler(sys.stdout))
        _set_handler(logger, "STDOUT", DEFAULT_PROGNAME)
        logger.setLevel(level)

        init_structlog()
        module.initialized = True

    return structlog.getLogger(LOGGER_NAME)


def setup_agent_logger(config):
    """
    Initialize the agent logger with configurations.
    :param config: instance of AgentConfig or dict
    :return: None
    """
    if config.get_value("agent.logger.stdout"):
        path = STDOUT
    elif config.get_value("agent.logger.stderr"):
        path = STDERR
    else:
        path = config.get_value("agent.logger.path")
    level = config.get_value("agent.logger.level").upper()

    logger = logging.getLogger(LOGGER_NAME)

    _set_logger_info(logger, config, path, level)

    cs_str.initialize_logger(structlog.getLogger(LOGGER_NAME))


def setup_security_logger(cfg):
    config = cfg if cfg else {}

    if not config.get_value("protect.enable"):
        return

    path = config.get_value("agent.security_logger.path")
    level = config.get_value("agent.security_logger.level").upper()

    logger = logging.getLogger(SECURITY_LOGGER_NAME)

    _set_level(logger, level)

    handler = _get_handler(path)
    logger.addHandler(handler)

    fmt = (
        f"%(asctime)s {get_hostname()} CEF:0|Contrast Security|Contrast Agent Python|{contrast.__version__}|"
        "SECURITY|%(message)s|%(level)s|'pri=%(rule_id)s' 'src=%(source_ip)s' 'spt=%(source_port)s' "
        "'request=%(request_url)s' 'requestMethod=%(request_method)s' 'app=%(application)s' "
        "'outcome=%(outcome)s'"
    )

    formatter = logging.Formatter(fmt)
    handler.setFormatter(formatter)


def security_log_msg(msg, app_name, rule_name, outcome):
    context = contrast.CS__CONTEXT_TRACKER.current()
    ip = port = url = method_name = "-"

    logger = logging.getLogger(SECURITY_LOGGER_NAME)

    if context is not None:
        ip = context.request.client_addr or "-"
        port = context.request.host_port
        method_name = context.request.method
        url = context.request.path

    log_context = dict(
        level=logger.level,
        rule_id=rule_name,
        source_ip=ip,
        source_port=port,
        request_url=url,
        request_method=method_name,
        application=app_name,
        outcome=outcome,
    )

    logger.warning(msg, extra=log_context)


def reset_agent_logger(log_path, log_level, config):
    """
    Reset agent logger path and/or level after the logger has already been created.

    Also note that progname is never reset so we use the one already set to the logger.

    :return: Bool if any logger value is reset
    """
    logger = logging.getLogger(LOGGER_NAME)

    is_reset = False
    current_path_option = config.get("agent.logger.path") if config else None
    # A configuration path can be changed if the config is unset or we're not set directly to STDOUT or STDERR
    changeable_path = not config or not (
        config.get_value("agent.logger.stdout")
        or config.get_value("agent.logger.stderr")
    )
    if log_path:
        need_update = current_path_option is None or (
            log_path != current_path_option.value()
            and current_path_option.source() in (CONTRAST_UI_SRC, DEFAULT_VALUE_SRC)
            and changeable_path
        )
        if current_path_option:
            current_path_option.ui_value = log_path
        if need_update:
            current_handler = logger.handlers[0]
            progname = current_handler.filters[0].progname
            _set_handler(logger, log_path, progname)
            # print so it shows up in STDOUT
            print(f"Contrast Agent Logger updated path to {log_path}")
            is_reset = True

    current_level_option = config.get("agent.logger.level") if config else None
    if log_level:
        need_update = current_level_option is None or (
            log_level != current_level_option.value()
            and current_level_option.source() in (CONTRAST_UI_SRC, DEFAULT_VALUE_SRC)
        )
        if current_level_option:
            current_level_option.ui_value = log_level
        if need_update:
            _set_level(logger, log_level)

            # print so it shows up in STDOUT
            print(f"Contrast Agent Logger updated level to {log_level}")
            # Avoid circular import
            from contrast.agent.agent_lib import update_log_options

            if update_log_options(log_level):
                print(f"Contrast Agent Lib Logger updated level to {log_level}")
            is_reset = True
    return is_reset


def _set_logger_info(logger, config, path, level):
    progname = config.get_value("agent.logger.progname")

    _set_handler(logger, path, progname)
    _set_level(logger, level)


def _set_handler(logger, path, progname):
    """
    A logger's handler is what determines where the log records will be printed to
    and what format they will have.

    To reset a handler, we delete the existing handlers and create a new one.

    CONTRAST-39746 defined the datetime format as ISO_8601. The one here is
    without ms as the logger doesn't natively support both ms and time zone at this time.
    """
    handler = _get_handler(path)
    program_filter = AgentFilter(progname=progname)
    handler.addFilter(program_filter)

    # empty all handlers so there is only one logging handler with this config
    logger.handlers = []
    logger.addHandler(handler)


def _get_handler(path):
    if path == STDOUT:
        handler = logging.StreamHandler(sys.stdout)
    elif path == STDERR:
        handler = logging.StreamHandler(sys.stderr)
    else:
        try:
            handler = logging.FileHandler(path)
        except Exception as e:
            print(e)
            # path could be '' or None
            handler = logging.StreamHandler()

    return handler


def _set_level(logger, level: str) -> None:
    if level.upper() == "TRACE":
        level = "DEBUG"
        print("Contrast Python Agent: TRACE logging is equivalent to DEBUG")
    try:
        logger.setLevel(level)
    except ValueError:
        # this fails validation if the level is an invalid value
        logger.setLevel(DEFAULT_LOG_LEVEL)


class AgentFilter(logging.Filter):
    def __init__(self, progname=None):
        self.progname = progname
        super().__init__()

    def filter(self, record):
        record.progname = self.progname
        return super().filter(record)



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
import re

from contrast.utils.configuration_utils import flatten_config, load_yaml_config
from contrast.utils.decorators import cached_property
from contrast_vendor import structlog as logging

from .agent import Agent
from .api import Api
from .application import Application
from .assess import Assess
from .inventory import Inventory
from .protect import Protect
from .root import Root
from .server import Server

logger = logging.getLogger("contrast")

DEFAULT_TEAMSERVER_TYPE = "EOP"
PUBLIC_SASS_TEAMSERVER_TYPES = {
    "app.contrastsecurity.com": "SAAS_DEFAULT",
    "eval.contrastsecurity.com": "SAAS_POV",
    "ce.contrastsecurity.com": "SAAS_CE",
    "app.contrastsecurity.jp": "SAAS_TOKYO",
    "cs": "SAAS_",
}

PRIVATE_SASS_TEAMSERVER_TYPE = "SAAS_CUSTOM"
PRIVATE_SASS_DOMAIN = ["contrastsecurity.com", "contrastsecurity.jp"]

TESTING_TEAMSERVER_TYPE = "TESTING"
TESTING_TEAMSERVER_NAMES = [
    "darpa",
    "alpha.contrastsecurity.com",
    "apptwo.contrastsecurity.com",
    "teamserver-staging.contsec.jp",
    "teamserver-staging.contsec.com",
    "security-research.contrastsecurity.com",
    "teamserver-ops.contsec.com",
]


class AgentConfig(object):
    def __init__(self):
        self.config_status = None
        self._config = {}
        self.build_configs()
        self.check_for_api_config()

    def get(self, key):
        return self._config.get(key, None)

    def get_value(self, key):
        option = self.get(key)
        return option.value() if option else None

    def get_loggable_value(self, key):
        option = self.get(key)
        return option.loggable_value() if option else None

    def keys(self):
        return self._config.keys()

    def get_loggable_config(self):
        """
        The purpose of this function is to return a copy of the effective config with their resolved values rather than
        the ConfigOption object.
        """

        loggable_config = {}
        for key in self._config:
            loggable_config[key] = self.get_loggable_value(key)

        return loggable_config

    def log_config(self):
        loggable_config = self.get_loggable_config()

        logger.info("Current Configuration", **loggable_config)

    def build_configs(self):
        """
        For each class representing different parts of the config (Agent, etc),
        take the current config and apply any overriding logic such as overriding with
        environment keys/values.

        At this time the order of precedence is:
            os.env > Config yaml

        Meaning that a config value defined in os.environ must be used instead of
        the same config defined in contrast_security.yaml

        Note that CLI args (sys.argv) are not supported. This may change if the
        agent becomes a runner.
        """
        yaml_filename = None
        yaml_config = load_yaml_config()
        if yaml_config:
            yaml_filename = yaml_config.get("loaded_configuration_filename", None)
            yaml_config = flatten_config(yaml_config)
        else:
            logger.info("No YAML config found; using default settings")

        builders = [Api, Agent, Application, Assess, Inventory, Protect, Root, Server]
        for builder in builders:
            status = builder().build(self._config, yaml_config, yaml_filename)
            if status:
                # An error here indicates a failure with the type cast of the value
                self.config_status = status

    def check_for_api_config(self):
        """
        Validate api configurations were set.

        If any api configs are missing, log at ERROR and disable the agent.

        Returns the missing values or an empty list if none were missing.
        """

        api_keys = [
            "api.url",
            "api.service_key",
            "api.api_key",
            "api.user_name",
        ]

        missing_values = []
        for key in api_keys:
            option = self.get(key)
            if not option or option.value() in (None, ""):
                missing_values.append(key)

        if missing_values:
            msg = (
                f"Missing a required connection value for: {', '.join(missing_values)}"
            )
            logger.error(msg)
            self.config_status = "Unable to connect to Contrast, insufficient connection properties provided."
            enable_option = self.get("enable")
            enable_option.override_value = False

        return missing_values

    @property
    def session_id(self):
        return self.get_value("application.session_id")

    @session_id.setter
    def session_id(self, session_id):
        session_id_option = self.get("application.session_id")
        session_id_option.ui_value = session_id

        logger.debug(
            "Set session_id to %s", session_id_option.value(), direct_to_teamserver=1
        )

    def get_session_metadata(self):
        return self.get_value("application.session_metadata")

    @property
    def app_code(self):
        return self.get_value("application.code")

    @property
    def app_metadata(self):
        return self.get_value("application.metadata")

    @property
    def app_group(self):
        return self.get_value("application.group")

    @property
    def app_tags(self):
        return self.get_value("application.tags")

    @property
    def assess_tags(self):
        return self.get_value("assess.tags")

    @cached_property
    def is_request_audit_enabled(self):
        return self.get_value("api.request_audit.enable")

    @cached_property
    def assess_enabled(self):
        return self.get_value("assess.enable")

    @cached_property
    def should_rewrite(self):
        return self.assess_enabled and self.get_value("agent.python.rewrite")

    @cached_property
    def should_apply_policy_rewrites(self):
        return self.should_rewrite and self.get_value(
            "agent.python.enable_policy_rewrites"
        )

    @cached_property
    def should_enable_sys_monitoring(self):
        return self.get_value("agent.python.enable_sys_monitoring")

    @cached_property
    def enable_automatic_middleware(self) -> bool:
        return self.get_value("agent.python.enable_automatic_middleware")

    @cached_property
    def application_activity_polling_interval(self):
        return self.get_value("agent.polling.app_activity_ms")

    @cached_property
    def server_settings_poll_interval(self):
        return self.get_value("agent.polling.server_settings_ms")

    @cached_property
    def heartbeat_poll_interval(self):
        return self.get_value("agent.polling.heartbeat_ms")

    @cached_property
    def teamserver_type(self):
        url = self.get_value("api.url")

        if not url:
            return ""

        for name, ts_type in PUBLIC_SASS_TEAMSERVER_TYPES.items():
            if name in url:
                # This regex matches https://cs001.contrastsecurity.com
                r_url = re.match(r".*(cs\d{3})\..*", url)
                if r_url:
                    # and gets the second group which is the matched in brackets cs001
                    # adds it to the SAAS_ type and becomes SAAS_CS001
                    numbers = r_url.group(1).upper()

                    return ts_type + numbers

                return ts_type

        for name in TESTING_TEAMSERVER_NAMES:
            if name in url:
                return TESTING_TEAMSERVER_TYPE

        for name in PRIVATE_SASS_DOMAIN:
            if name in url:
                return PRIVATE_SASS_TEAMSERVER_TYPE

        return DEFAULT_TEAMSERVER_TYPE



# -*- coding: utf-8 -*-
# Copyright © 2023 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from contrast.agent.assess.rules.config.base_flask_config_rule import (
    BaseFlaskConfigRule,
)
from contrast.agent.assess.rules.config.session_age_rule import SessionAgeRuleMixin


class FlaskSessionAgeRule(SessionAgeRuleMixin, BaseFlaskConfigRule):
    SETTINGS_VALUE = "PERMANENT_SESSION_LIFETIME"
